#ifndef GLBASEEVALUATOR_H_
#define GLBASEEVALUATOR_H_

class GLBaseFitness;
class GLBaseOrganism;
/**
 * Base evaluator for all organisms.
 */
class GLBaseEvaluator
{
public:
  /**
   * Simple constructor.
   */
  GLBaseEvaluator()
    {resetCounter();}

  /**
   * Destructor.
   */
  virtual ~GLBaseEvaluator() {};

  /**
   * Evaluates fitness function of the organism.
   * @param organism organism to be evaluated.
   * @return fitness value storage.
   */
  virtual GLBaseFitness* evaluateOrganism(GLBaseOrganism* organism) = 0;

  /**
   * Returns the current value of the counter.
   * @return the current value of the counter.
   */
  long getCounter() const
    { return m_counter; }

  /**
   * Returns the number of objectives in fittness function. Makes it
   * possible to distinguish between single and multiple objective
   * optimisation even if the same GA is used for optimisation.
   */
  virtual int getNumberOfObjectives() const = 0;

  /**
   * Increases the value of the internal counter.
   */
  void increaseCounter()
    { ++m_counter; }
  
  /**
   * Sets the internal counter to zero.
   */
  void resetCounter()
    { m_counter = 0; }
  
private:
  long m_counter;  //to keep track on how many times the evaluator was called.
                  //if necessary. Normally to be increased when
                  //evaluateOrganism is called.
};

#endif /*GLBASEEVALUATOR_H_*/
