#include <NLstl_utils.h>

#include "GLBaseGaStatus.h"
#include "GLConstants.h"
#include "GLUtils.h"

/**
 * Constructor. Sets the status to the initial state.
 */
GLBaseGaStatus::GLBaseGaStatus():
      m_timer("", false)
{
  m_current_state = GLConstants::GA_STATE_INITIALISED;
  m_best_ever_organisms.clear();
  m_current_generation = 0;
  m_evaluator_calls = 0;
  m_generations_since_solution_improvement = 0;
  m_generations_since_new_best_found = 0;
  m_last_generation.clear();
  m_time_required = 0;
}//constructor

/**
 * Copying constructor.
 * @param source source of the data.
 */
GLBaseGaStatus::GLBaseGaStatus(const GLBaseGaStatus* source)
{
  m_current_state = source-> m_current_state;
  GLUtils::makeHardCopy(source->m_best_ever_organisms, 
                        m_best_ever_organisms);
  GLUtils::makeHardCopy(source->m_last_generation, 
                        m_last_generation);
  m_current_generation = source->m_current_generation;
  m_evaluator_calls = source->m_evaluator_calls;
  m_generations_since_solution_improvement = 
                         source->m_generations_since_solution_improvement;
  m_generations_since_new_best_found =
                         source->m_generations_since_new_best_found;
  m_time_required = source->m_time_required;
  m_timer = source->m_timer;
}//constructor


/**
 * Destructor. Destroys organisms from the last population.
 */
GLBaseGaStatus::~GLBaseGaStatus()
  {
    for_each(m_best_ever_organisms.begin(), m_best_ever_organisms.end(), 
             nlutils::DeleteObject());
    for_each(m_last_generation.begin(), m_last_generation.end(), 
             nlutils::DeleteObject());
    m_best_ever_organisms.clear();
    m_last_generation.clear();
  }//destructor

/**
 * Method for output the status to stream.
 * @param out destination stream. 
 */
ostream& GLBaseGaStatus::toStream(ostream& out) const
  {
    //print the status of GA
    out << "GA current state:" << m_current_state << " (" 
        << GLConstants::toStringGaState(m_current_state) << ")" << endl;
    out << "--------- Last generation (if kept and GA stopped) ---------\n";
    out << "Size: " << m_last_generation.size() 
        << endl;
    GLUtils::printVectorOfOrganisms(out, m_last_generation);
    out << "Current generation: " << m_current_generation << endl;
    out << "Generation since solution improved: " 
        << m_generations_since_solution_improvement << endl;
    out << "Generation since new best was found: " 
        << m_generations_since_new_best_found << endl;
    out << "Number of evaluator calls: " 
        << m_evaluator_calls << endl;
    //print the best ever organisms
    out << "--------- Best ever organisms ---------\n";
    out << "Number or organisms: "
        << m_best_ever_organisms.size() << endl;
    GLUtils::printVectorOfOrganisms(out, m_best_ever_organisms);
    return out;
  }//toStream
