#ifndef GLBASEGASTATUS_H_
#define GLBASEGASTATUS_H_
#include <vector>
#include <NLInterfaceOutput.h>
#include <NLTimer.h>

#include "GLBaseOrganism.h"
using namespace std;
/**
 * Class which contains the current status of the GA. This is again mostly
 * the data structure, so it has all the parameters open. Its content should
 * be modified during the GA run.
 */
struct GLBaseGaStatus: public nlutils::NLInterfaceOutput
{
public:
  /**
   * Constructor. Sets the status to the initial state.
   */
  GLBaseGaStatus();

  /**
   * Copying constructor.
   * @param source source of the data.
   */
  GLBaseGaStatus(const GLBaseGaStatus* source);

  /**
   * Destructor. Destroys everything.
   */
  virtual ~GLBaseGaStatus();

  /**
   * Returns pointer to the copy of the current status.
   */
  virtual GLBaseGaStatus* copy() const
  { return new GLBaseGaStatus(this); }
  
  /**
   * Method for output the status to stream.
   * @param out destination stream. 
   */
  virtual ostream& toStream(ostream& out) const;
public:
  ///Current status of GA
  int m_current_state;
  ///best ever organism(s) if more then one have the same fittness function
  ///or for Pareto front. Hard clones of the organisms in population,
  ///so they should survive the destruction of parent population.
  vector<const GLBaseOrganism*> m_best_ever_organisms;
  ///current generation number
  int m_current_generation;
  ///how many times evaluator called during the GA run (evaluator must
  ///track its calls).
  long m_evaluator_calls;
  ///number of generations since the solution was improved -- either new
  ///fitter organism found or additional point on Pareto front.
  int m_generations_since_solution_improvement;
  ///number of generations since new best solution was found (fitter or
  ///having the same fittness as already found best organism).
  ///it should be always <= m_generations_since_solution_improvement.
  ///For MultiObjective optimisation those 2 variables should probably 
  ///be the same.
  int m_generations_since_new_best_found;
  ///Vector of organisms in last generation(s) for GA (after GA is done).
  ///They are hard clones of the last population, so they will survivie
  ///the population's destruction.
  vector<GLBaseOrganism*> m_last_generation;
  
  ///Time required for optimization (set after optimization is over) in
  ///seconds.
  double m_time_required;
  
  ///To be used to mesure the time required for optimisation. It is 
  ///responsibility of the GA algorithm to reset the timer befor optimization
  ///starts.
  nlutils::NLTimer m_timer;
};
#endif /* GLBASEGASTATUS_H_ */
