#include "GLBaseMutator.h"
#include "GLLogicError.h"

  
/**
 * Copying constructor.
 * @param mutator source to copy from.
 */
GLBaseMutator::GLBaseMutator(const GLBaseMutator& mutator)
  {
    m_mutation_rate = mutator.m_mutation_rate;
  }//Constructor


/**
 * Resets the mutator (e.g. sets counter, if any, to zero etc.).
 */
void GLBaseMutator::reset()
  {
    
  }//reset

/**
 * Sets the new mutation rate for the mutator. 
 * @param mutation_rate rate of the mutation, interpretation is upto
 *    concrete implementation of the mutator. Normally one would expect
 *    to keep it in (0; 1] interval.
 * @param check_range if true, check is made that mutation rate is
 *    > 0 and < 1.
*/
 void GLBaseMutator::setMutationRate(double mutation_rate, 
                   bool check_range)
   {
     if (check_range && ((mutation_rate <= 0) || (mutation_rate > 1)))
       {
         char errormsg[1000];
         sprintf(errormsg, "\nGLBaseMutator::setMutationRate\n"
                           "Range check is on and mutationRate (= %e)"
                           "is outside the (0; 1) interval\n",
                           mutation_rate);
         throw GLLogicError(errormsg);
       }
    m_mutation_rate = mutation_rate;
   }//setMutationRate
