#ifndef GLBASEPARAMETERS_H_
#define GLBASEPARAMETERS_H_

/**
 * Base class for passing the parameters to GL classes -- allows to
 * use the GLFactory for instantiating classes with different constructor
 * parameters.
 * The classes from the GLParameters family should provide public access to
 * its members as they really are just structures with parameters.
 * To underline it they are declared using 'struct', not 'class'.
 * It is strongly recommended, that each of Parameters class provide
 * default values to its members through setDefaultValues() method.
 */
struct GLBaseParameters
{
public:
  /**
   * Simple constructor.
   */
  GLBaseParameters() {};

  /**
   * Simple destructor.
   */
  virtual ~GLBaseParameters() {};

  /**
   * Creates the copy of the parameters.
   * @return pointer to identical copy of the current class exemplar.
   */
  virtual GLBaseParameters* copy() const = 0;
};

#endif /*GLBASEPARAMETERS_H_*/
