/*
 * GLBaseVariator.h
 *
 *  Created on: 31-Dec-2008
 *      Author: Artem_Babayan
 */

#ifndef GLBASEVARIATOR_H_
#define GLBASEVARIATOR_H_
#include "GLConstants.h"

class GLBaseMutator;
class GLBaseOrganismStorage;
class GLBasePopulation;
/**
 * Class for applying the mutator to the container of organisms.
 */
class GLBaseVariator
{
public:
  /**
   * Constructor. Takes the pointer to mutator operator.
   * @param mutator mutator operator.
   */
  GLBaseVariator(GLBaseMutator *mutator);

  /**
   * Deletes all, including mutator operator.
s  */
  virtual ~GLBaseVariator();

  /**
   * Applies mutator to the set container of organisms.
   * @param population population to which mutated organisms belong -- mainly
   *    to check for uniqueness of the mutants.
   * @param organisms container with organisms.
   * @param mutants where mutated organisms will be added.
   * @param fill when equal to APPEND the container with mutants will be
   *    appended with new organisms, when REPLACE - mutants will be emptied
   *    at the beginning.
   * @param twins when NO_TWINS only unique organisms will be added.
   * @param max_mutants maximum number of mutants to be created,
   *            if < 0 -- number is limited only by the mutation algorithm
   *            used.
   * @return number of new organisms produced by this method.
   */
  virtual int applyMutator(GLBasePopulation* population,
                   GLBaseOrganismStorage* organisms,
                   GLBaseOrganismStorage* mutants,
                   GLConstants::TStorageFilling fill =
                     GLConstants::APPEND,
                   GLConstants::TTwinsFlags twins =
                     GLConstants::NO_TWINS,
                   int max_mutants = -1) = 0;

  /**
   * Returns the mutator associated with this variator.
   * @return the mutator registered with this variator.
   */
  GLBaseMutator* getMutator()
  { return m_mutator; }

  /**
   * Returns the mutator associated with this variator.
   * @return the mutator registered with this variator.
   */
  const GLBaseMutator* getMutator() const
  { return m_mutator; }
  
  /**
   * Resets the variator (e.g. sets counter, if any, to zero etc.) and its
   * mutator.
   */
  virtual void reset();
protected:
  ///mutation operator. Gets destroyed with this class.
  GLBaseMutator* m_mutator;
};

#endif /* GLBASEVARIATOR_H_ */
