#include "GLConstants.h"

/**
 * Returns string representing the value of bounds.
 * Useful for debug.
 * @param bounds constant representing bounds of the gene.
 * @return string corresponding to name of the constant.
 */
string GLConstants::toString (TGeneBounds bounds)
  {
    switch(bounds)
    {
      case BOUNDED_GENES:
          return "BOUNDED_GENES";
      case UNBOUNDED_GENES:
          return "UNBOUNDED_GENES";
      default:
          return "--- unknown TGeneBounds value ---";
    }

  }//toString

/**
 * Returns string representing the value of limits constant.
 * Useful for debug.
 * @param limits constant representing limits type of the object.
 * @return string corresponding to name of the constant.
 */
string GLConstants::toString (TSizeLimits limits)
  {
    switch(limits)
    {
      case FIXED:
          return "FIXED";
      case NOT_FIXED:
          return "NOT_FIXED";
      default:
          return "--- unknown TSizeLimits value ---";
    }
  }//toString


/**
 * Return string representing the memory policy. Using for debug output.
 * @param memory constant for memory policy.
 * @return string corresponding to name of constant.
 */
string GLConstants::toString (TMemoryPolicy memory)
  {
    switch (memory)
    {
      case DO_NOT_FREE:
            return "DO_NOT_FREE";
      case FREE_ALL:
            return "FREE_ALL";
      case FREE_UNLOCKED:
            return "FREE_UNLOCKED";
      default:
            return "--- unknown TMemoryPolicy value ---";
    }
  }//toString


/**
 * Returns string representing the value of the breeder type constant.
 * Useful for debug.
 * @param breeder_type constant for breeder type.
 * @return string which is the name for the constant.
 */
string GLConstants::toStringBreeder(int breeder_type)
  {
    switch(breeder_type)
    {
      case BREEDER_STANDARD:
        return "BREEDER_STANDARD";
      default:
        return "BREEDER_UNKNOWN";
    }
  }//toStringBreeder

/**
 * Returns string representing the value of the crossover type constant.
 * Useful for debug.
 * @param crossover_type constant for crossover type.
 * @return string which is the name for the constant.
 */
string GLConstants::toStringCrossover(int crossover_type)
  {
    switch(crossover_type)
    {
      case CROSSOVER_VECTOR_KEEP_MATCHING_INT:
        return "CROSSOVER_VECTOR_KEEP_MATCHING_INT";
      default:
        return "CROSSOVER_UNKNOWN";
    }
  }//toStringCrossover

/**
 * Returns string representing the value of the current GA state.
 * Useful for debug.
 * @param ga_state constant for GA state type.
 * @return string which is the name for the constant. 
 */
string GLConstants::toStringGaState(int ga_state)
  {
    switch(ga_state)
    {
      case GA_STATE_INITIALISED:
        return "GA_STATE_INITIALISED";
      case GA_STATE_RUNNING:
        return "GA_STATE_RUNNING";
      case GA_STATE_STOPPED_ERROR:
        return "GA_STATE_STOPPED_ERROR";
      case GA_STATE_STOPPED_NORMALLY:
        return "GA_STATE_STOPPED_NORMALLY";
      default:
        return "UNKNOWN STATE";
    }//swithc ga_state
  }//toStringGaState

/**
 * Returns string representing the value of the organism initialiser type 
 * constant.
 * Useful for debug.
 * @param initialiser_type constant for organism initialiser type.
 * @return string which is the name for the constant.
 */
string GLConstants::toStringInitialiser(int initialiser_type)
  {
    switch(initialiser_type)
    {
      case INITIALISER_SHUFFLE_INT_VECTOR:
        return "INITIALISER_SHUFFLE_INT_VECTOR";
      default:
        return "INITIALISER_UNKNOWN";
    }
  }//toStringPopulation


/**
 * Returns string representing the value of the mutator type constant.
 * Useful for debug.
 * @param mutator_type constant for mutator type.
 * @return string which is the name for the constant.
 */
string GLConstants::toStringMutator(int mutator_type)
  {
    switch(mutator_type)
    {
      case MUTATOR_GENES_SWAPPING:
        return "MUTATOR_GENES_SWAPPING";
      case MUTATOR_GENES_SHUFFLING:
        return "MUTATOR_GENES_SHUFFLING";
      case MUTATOR_GENE_MOVING:
        return "MUTATOR_GENE_MOVING";
      case MUTATOR_COMBINED:
        return "MUTATOR_COMBINED";
      default:
        return "MUTATOR_UNKNOWN";
    }
  }//toStringMutator

/**
 * Returns string representing the value of the population type constant.
 * Useful for debug.
 * @param population_type constant for population type.
 * @return string which is the name for the constant.
 */
string GLConstants::toStringPopulation(int population_type)
  {
    switch(population_type)
    {
      case POPULATION_STANDARD:
        return "POPULATION_STANDARD";
      default:
        return "POPULATION_UNKNOWN";
    }
  }//toStringPopulation

/**
 * Returns string representing the value of the ranker type constant.
 * Useful for debug.
 * @param ranker_type constant for ranker type.
 * @return string which is the name for the constant.
 */
string GLConstants::toStringRanker(int ranker_type)
  {
    switch(ranker_type)
    {
      case RANKER_STANDARD_SO:
        return "RANKER_STANDARD_SO";
      default:
        return "RANKER_UNKNOWN";
    }
  }//toStringRanker

/**
 * Returns string representing the value of the selectionist type constant.
 * Useful for debug.
 * @param selectionist_type constant for selectionist type.
 * @return string which is the name for the constant.
 */
string GLConstants::toStringSelectionist(int selectionist_type)
  {
    switch(selectionist_type)
    {
      case SELECTIONIST_ELITIST:
        return "SELECTIONIST_ELITIST";
      case SELECTIONIST_MIXER:
        return "SELECTIONIST_MIXER";
      default:
        return "SELECTIONIST_UNKNOWN";
    }
  }//toStringSelectionist

/**
 * Returns string representing the value of the variator type constant.
 * Useful for debug.
 * @param variator_type constant for variator type.
 * @return string which is the name for the constant.
 */
string GLConstants::toStringVariator(int variator_type)
  {
    switch(variator_type)
    {
      case VARIATOR_STANDARD:
        return "VARIATOR_STANDARD";
      default:
        return "VARIATOR_UNKNOWN";
    }
  }//toStringVariator
