#include <NLstl_utils.h>

#include "GLCrossoverOnePoint.h"
#include "GLInterfaceSwapTails.h"
#include "GLLogicError.h"
#include "GLRandomNumbersGenerator.h"
/**
 * Creates an 2 offsprings from 2 parents by randomly swapping tails.
 * @param parents vector of parents.
 * @param children vector of new organisms obtained from parents. Previous
 *    content will not be preserved.
 */
void GLCrossoverOnePoint::makeOffsprings(
        const vector<GLBaseOrganism*>& parents,
        vector<GLBaseOrganism*>& children)
  {
    //Check, that 2 parents are passed.
    if (parents.size() != 2)
    {
          char errormsg[1000];
          sprintf(errormsg, 
              "\nGLCrossoverOnePoint<T>::makeOffspring\n"
              "Can accept 2 parents only, while vector of %i parents "
              "was passed\n",
              (int)parents.size());
          throw GLLogicError(errormsg);
    }
    //find the position at which tails will be swapped
    int min_size = min(parents[0]->getSize(), parents[1]->getSize());
    int pos = GLRandomNumbersGenerator::getRandomInteger(min_size);
    
    //create clones.
    children.clear();
    children[0] = parents[0]->makeClone();
    children[1] = parents[1]->makeClone();
    
    //swap the tails
    GLInterfaceSwapTails *swapper = 
        nlutils::transform_pointer<GLInterfaceSwapTails,
                                   GLBaseOrganism>(children[0], 
                           "GLCrossoverOnePoint::makeOffspring\n"
                           "one of parents do not implement the "
                           "InterfaceSwapTails\n");
    swapper->swapTails(children[1], pos);
  }//makeOffspring
