#include <NLstl_utils.h>

#include "GLCrossoverTwoPoints.h"
#include "GLInterfaceSwapSection.h"
#include "GLLogicError.h"
#include "GLRandomNumbersGenerator.h"

/**
 * Creates an 2 offsprings from 2 parents by swapping sections bounded
 * by randomly chosen 2 points.
 * @param parents vector of parents. Must contain only 2 elements each of
 *    them implementing GLInterfaceSwapSections.
 * @param children vector of new organisms obtained from parents. Previous
 *    content will not be preserved.
 */
void GLCrossoverTwoPoints::makeOffsprings(
            const vector<GLBaseOrganism*>& parents,
            vector<GLBaseOrganism*>& children)
  {
    //Check, that 2 parents are passed.
    if (parents.size() != 2)
    {
          char errormsg[1000];
          sprintf(errormsg, 
              "\nGLCrossoverTwoPoints<T>::makeOffspring\n"
              "Can accept 2 parents only, while vector of %i parents "
              "was passed\n",
              (int)parents.size());
          throw GLLogicError(errormsg);
    }
    //find the position at which section will be swapped
    int min_size = min(parents[0]->getSize(), parents[1]->getSize());
    int leftPos = GLRandomNumbersGenerator::getRandomInteger(min_size - 1);
    int rightPos = 
          GLRandomNumbersGenerator::getRandomInteger(leftPos + 1, min_size);
    
    //create clones.
    children.clear();
    children[0] = parents[0]->makeClone();
    children[1] = parents[1]->makeClone();
    
    //swap the sections
    GLInterfaceSwapSection *swapper = 
        nlutils::transform_pointer<GLInterfaceSwapSection,
                                   GLBaseOrganism>(children[0], 
                           "GLCrossoverOnePoint::makeOffspring\n"
                           "first parent does not implement the "
                           "InterfaceSwapSection\n");
    swapper->swapSections(children[1], leftPos, rightPos);
  }//makeOffspring
