#ifndef GLCROSSOVERVECTORKEEPMATCHING_H_
#define GLCROSSOVERVECTORKEEPMATCHING_H_
#include "GLBaseCrossover.h"
/**
 * Crossover which is to be applied to vector-based organisms. Takes 2
 * organisms of type GLOrganismVector<T> to breed and do the following:
 * 1) For the i: org1[i] == org2[i] do nothing.
 * 2) Shuffle randomly j: org1[j] != org2[j]
 * It is assumed, that both organisms consist of the same set of genes which
 * are shuffled randomly.
 */
template <class T>
class GLCrossoverVectorKeepMatching: public GLBaseCrossover
{
public:
  /**
   * Simple constructor.
   */
  GLCrossoverVectorKeepMatching() {};

  /**
   * Destructor, destroys everything.
   */
  virtual ~GLCrossoverVectorKeepMatching() {};

  /**
   * Creates an offspring from the vector of parents. Takes 2 vector-based
   * organisms and creates children by keeping matching genes in their place
   * and randomly shuffling others.
   * @param parents vector of parents.
   * @param children vector of new organisms obtained from parents. Previous
   *    content will not be preserved.
   */
  virtual void makeOffsprings(const vector<GLBaseOrganism*>& parents,
                                   vector<GLBaseOrganism*>& children);
                    
};

#endif /* GLCROSSOVERVECTORKEEPMATCHING_H_ */
