#include "GLFitness.h"
#include "GLLogicError.h"

/**
  * Constructor to store the value of the fittness function.
  * @param value fitness value.
 */
template<class TFitness>
GLFitness<TFitness>::GLFitness(const TFitness& value):
  GLBaseFitness()
{
  setValue(value);
}//Constructor

/**
 * Copying constructor.
 * @param source sourse of the fitness value.
 */
template<class TFitness>
GLFitness<TFitness>::GLFitness(const GLFitness<TFitness>& source):
  GLBaseFitness()
  {
    m_value = source.m_value;
  }//Constructor

/**
 * Returns true if 2 fitness values are equal.
 */
template<class TFitness>
bool GLFitness<TFitness>::isEqual(const GLBaseFitness* fitness) const
{    
  //first check if another fitness can be compared with this one
  //in principle
  const GLFitness<TFitness>* fit = 
       dynamic_cast<const GLFitness<TFitness>*>(fitness);
  if (fit == NULL) return false;
  return m_value == fit->m_value;
}//isEqual


/**
 * Returns true if this fitness is less then the one passed as
 * a parameter.
 * @param fitness pointer to fitness to make a comparison with.
 */
template<class TFitness>
bool GLFitness<TFitness>::isLess(const GLBaseFitness* fitness) const
{
  //check, if 2 classes are comparable
  const GLFitness<TFitness>* fit = 
       dynamic_cast<const GLFitness<TFitness>*>(fitness);
  if (fit == NULL)
    {
      char errormsg[1000];
      sprintf(errormsg, "\nGLFitness<TFitness>::isLess\n"
                        "Attempt to compare with not-compatible class\n");
      throw GLLogicError(errormsg);
    }
  return m_value < fit->m_value;
}//isLess
