#ifndef GLFITNESS_H_
#define GLFITNESS_H_

#include "GLBaseFitness.h"
/**
 * Class for storage the fitness information. It is highly desirable, that
 * TFitness type was 'comparable' so it can be used in comparison operations
 * for organisms ranking etc.
 */
template <class TFitness>
class GLFitness : public GLBaseFitness
{
public:
  /**
   * Constructor to store the value of the fittness function.
   * @param value fitness value.
   */
   GLFitness(const TFitness& value);

   /**
    * Copying constructor.
    * @param source sourse of the fitness value.
    */
   GLFitness(const GLFitness<TFitness>& source);

  /**
   * Simple destructor.
   */
   virtual ~GLFitness() {};

   /**
    * Returns the copy of the fittness class.
    * @return the copy of this class.
    */
   virtual GLBaseFitness* copy() const
   { return new GLFitness<TFitness>(*this); }

   /**
    * Returns the fitness value.
    * @return the fitness value.
    */
   const TFitness& getValue() const
   { return m_value; }

   /**
    * Returns true if 2 fitness values are equal.
    */
   virtual bool isEqual(const GLBaseFitness* fitness) const;

   /**
    * Returns true if this fitness is less then the one passed as
    * a parameter.
    * @param fitness pointer to fitness to make a comparison with.
    */
   virtual bool isLess(const GLBaseFitness* fitness) const;

   /**
    * Sets the value of the fitness.
    * @param value new value of the fitness.
    */
   void setValue(const TFitness& value)
   { m_value = value; }
private:
  ///Fitness value.
  TFitness m_value;
};

/**
 * Predefined simple common types.
 */
typedef GLFitness<double> GLFitnessSingleDouble;
typedef GLFitness<int> GLFitnessSingleInt;
#endif /*GLFITNESS_H_*/
