#ifndef GLFITNESSSINGLEOBJECTIVE_H_
#define GLFITNESSSINGLEOBJECTIVE_H_
#include "GLFitness.h"

/**
 * The only difference between this class and its parent (GLFitness) that
 * it makes usable methods assignTo inherited from GLBaseFitness
 * as well stream output methods toStream and toStreamDebug.
 * It allows to collect statistics from multiple runs. It assumes, that
 * objective value
 *      a) can be converted to double.
 *      b) can be set to output stream.
 * The abovementioned methods will return pointer to
 * GLFitnessSingleObjective<double>.
 */
template <class TFitness>
class GLFitnessSingleObjective : public GLFitness<TFitness>
{
public:
  /**
   * Constructor to store the value of the fitness function.
   * @param value fitness value.
   */
   GLFitnessSingleObjective(const TFitness& value):
     GLFitness<TFitness>(value)
     {}

   /**
    * Copying constructor.
    * @param source sourse of the fitness value.
    */
   GLFitnessSingleObjective(const GLFitness<TFitness>& source):
     GLFitness<TFitness>(source)
     {}

  /**
   * Simple destructor.
   */
   virtual ~GLFitnessSingleObjective();

   /**
    * Assigns the value of fitness function GLFitnessSingleObjective<double>.
    * Note, that it means, that template parameter TFitness must support
    * transformation to double.
    * @param result should be compatible with type 
    *   GLFitnessSingleObjective<double>. E.g. this function may be
    *   used to transform GLFitness<int> to GLFitnessSingleObjective<double>.
    */
   virtual void assignTo(GLBaseFitness& result) const;

   /**
    * Returns the copy of the fittness class.
    * @return the copy of this class.
    */
   virtual GLBaseFitness* copy() const
   { return new GLFitnessSingleObjective<TFitness>(*this); }

   /**
    * Outputs the content of the organism to output stream.
    * @param out output stream.
    */
   virtual std::ostream& toStream(std::ostream& out) const
     { out << this->getValue(); return out; }

   /**
    * Outputs the content of the organism to output stream in
    * debug mode.
    * @param out output stream.
    */
   virtual std::ostream& toStreamDebug(std::ostream& out) const
     { return toStream(out); }
};

#endif /*GLFITNESSSINGLEOBJECTIVE_H_*/
