#include <stdlib.h>
#include <NLstl_utils.h>

#include "GLBaseOrganism.h"
#include "GLConstants.h"
#include "GLGaStatusStandard.h"
#include "GLUtils.h"

/**
 * Constructor. Initialises the status with initial data.
 */
GLGaStatusStandard::GLGaStatusStandard():
    GLBaseGaStatus()
{
  m_best_organisms_in_population.clear();
  m_generations_since_population_changed = 0;
}//constructor

/**
 * Copying constructor.
 * @param source source of data.
 */
GLGaStatusStandard::GLGaStatusStandard(const GLGaStatusStandard* source):
  GLBaseGaStatus(source)
{
  //only pointers are copied here
  m_best_organisms_in_population = source->m_best_organisms_in_population;
  m_generations_since_population_changed =
        source->m_generations_since_population_changed;
}//constructor

/**
 * Destructor -- destroys everything.
 */
GLGaStatusStandard::~GLGaStatusStandard()
{
}//destructor;

/**
 * Method for output the status to stream.
 * @param out destination stream. 
 */
ostream& GLGaStatusStandard::toStream(ostream& out) const
  {
    //print the  parent first
    GLBaseGaStatus::toStream(out);
    out << "Generations since population changed: " 
        << m_generations_since_population_changed << endl;
    if (m_current_state == GLConstants::GA_STATE_RUNNING)
      {
        out << "--------- Best organisms in current population ---------\n";
        out << "Number or organisms: "
            << m_best_organisms_in_population.size() << endl;
        GLUtils::printVectorOfOrganisms(out, m_best_organisms_in_population);
      }
    return out;
  }//toStream
