#ifndef GLGASTATUSSTANDARD_H_
#define GLGASTATUSSTANDARD_H_
#include <vector>
#include "GLBaseGaStatus.h"

using namespace std;
class GLBaseOrganism;
/**
 * Standard information about status of the single objective, 
 * single-population GA.
 */
struct GLGaStatusStandard : public GLBaseGaStatus
{
public:
  /**
   * Constructor. Initialises the status with initial data.
   */
  GLGaStatusStandard();

  /**
   * Copying constructor.
   * @param source source of data.
   */
  GLGaStatusStandard(const GLGaStatusStandard* source);

  /**
   * Destructor -- destroys everything.
   */
  virtual ~GLGaStatusStandard();

  /**
   * Returns pointer to the copy of the current status.
   */
  virtual GLBaseGaStatus* copy() const
  { return new GLGaStatusStandard(this); }
  
  /**
   * Method for output the status to stream.
   * @param out destination stream. 
   */
  virtual ostream& toStream(ostream& out) const;

public:
  ///best organism(s) in population -- pointers to the population's organisms.
  ///Again, they are either organisms(s) with minimal fittness of Pareto 
  ///front.
  vector<const GLBaseOrganism*> m_best_organisms_in_population;
  ///number of generations since the last time the population was changed
  ///(new organisms appeared there)
  int m_generations_since_population_changed;
};

#endif /* GLGASTATUSSTANDARD_H_ */
