#ifndef GLGASTOPPERMAXGENERATIONS_H_
#define GLGASTOPPERMAXGENERATIONS_H_

#include "GLBaseGaStopper.h"

/**
 * Class which stops the GA once it reached the maximum number of 
 * generations. This class requires, that status passed to the
 * stop method was actually GLGaStatusStandard.
 */
class GLGaStopperMaxGenerations: public GLBaseGaStopper
{
public:
  /**
   * Constructor.
   * @param max_generations number of generations to run before stop.
   */
  GLGaStopperMaxGenerations(int max_generations = 100);
  
  /**
   * Simple GA stopper.
   */
  virtual ~GLGaStopperMaxGenerations() {};
  
  /**
   * Method which determines whether it is time to stop a GA based on its
   * status.
   * @param ga_status current status of the ga (should point to the exemplar
   *  of GLGaStatusStandard class).
   * @return true when GA should stop.
   */
  virtual bool stop(const GLBaseGaStatus* ga_status);

private:
  int m_max_generations;  //number of generations to run before GA stop.
};

#endif /*GLGASTOPPERMAXGENERATIONS_H_*/
