#include <NLstl_utils.h>

#include "GLGaStatusStandard.h"
#include "GLGaStopperNoNew.h"


/**
 * Constructor.
 * @param max_no_new maximum number of generations without new organisms
 *    added to population before this stopper will be triggered.
 */
GLGaStopperNoNew::GLGaStopperNoNew(int max_no_new)
{
  m_max_no_new = max_no_new;
}//constructor

/**
 * Method which determines whether it is time to stop a GA based on its
 * status.
 * @param ga_status current status of the ga (should point to the exemplar
 *  of GLGaStatusStandard class).
 * @return true when GA should stop.
 */
bool GLGaStopperNoNew::stop(const GLBaseGaStatus* ga_status)
  {
    const GLGaStatusStandard* status = 
      nlutils::transform_pointer<const GLGaStatusStandard, 
                                 const GLBaseGaStatus>
        (ga_status, "GLGaStopperNoNew::stop\n"
                    "Passed status is not a pointer to GLGaStatusStandard\n");
    return (status->m_generations_since_population_changed >= m_max_no_new); 
  }//stop
