#ifndef GLGASTOPPERNONEW_H_
#define GLGASTOPPERNONEW_H_

#include "GLBaseGaStopper.h"
/**
 * Stopper which works when the population was not updated for a few 
 * generations. Requires pointer to GLGaStopperStandard.
 */
class GLGaStopperNoNew: public GLBaseGaStopper
{
public:
  /**
   * Constructor.
   * @param max_no_new maximum number of generations without new organisms
   *    added to population before this stopper will be triggered.
   */
  GLGaStopperNoNew(int max_no_new = 10);
  
  /**
   * Simple destructor to kill everything.
   */
  virtual ~GLGaStopperNoNew() {};

  /**
   * Method which determines whether it is time to stop a GA based on its
   * status.
   * @param ga_status current status of the ga (should point to the exemplar
   *  of GLGaStatusStandard class).
   * @return true when GA should stop.
   */
  bool stop(const GLBaseGaStatus* ga_status);

private:
  int m_max_no_new;
};

#endif /*GLGASTOPPERNONEW_H_*/
