#include "NLstl_utils.h"

#include "GLBaseMutator.h"
#include "GLBaseOrganism.h"
#include "GLInitialiserMutator.h"

 /**
  * Constructor. Note, that copies of initialOrganism and mutator are
  * stored in the initialiser.
  * @param initialOrganism base organism for future mutations.
  * @param mutator mutation operator to be applied to base organism.
  *   it is recommended, that this operator had a mutation rate 1.0.
  * @param returnInitial when true the first time the initialiseOrganism
  *   is called the initial organism will be returned. 
  */
 GLInitialiserMutator::GLInitialiserMutator(GLBaseOrganism *initialOrganism,
                      GLBaseMutator* mutator,
                      bool returnInitial)
   {
     nlutils::assertPointer(initialOrganism, 
           "initialOrganism from "
           "GLInitialiserMutator::GLInitialiserMutator");
     nlutils::assertPointer(mutator, 
           "mutator from "
           "GLInitialiserMutator::GLInitialiserMutator");
     m_initial_organism = initialOrganism->makeClone();
     m_mutator = mutator->copy();
     m_return_initial = returnInitial;
   }//constructor
 
 /**
  * Destructor. Destroys copies of initialOrganism and mutator stored
  * within this class.
  */
 GLInitialiserMutator::~GLInitialiserMutator()
   {
     if (m_initial_organism != NULL) delete m_initial_organism;
     if (m_mutator != NULL) delete m_mutator;
   }//destructor
 
 /**
  * Creates and initialise organism.
  * @return pointer to the organism initialised with some initial value.
  */
 GLBaseOrganism* GLInitialiserMutator::initialiseOrganism()
   {
     if (m_return_initial) 
       {
         m_return_initial = false;
         return m_initial_organism->makeClone();
       }
     GLBaseOrganism *organism = m_initial_organism->makeClone();
     m_mutator->mutateOrganism(organism);
     return organism;
   }

 /**
  * Next call of initialiseOrganism after this function will return
  * the initial organisim.
  */
 void GLInitialiserMutator::resetInitial()
   {
     m_return_initial = true;
   }
