#ifndef GLINITIALISERMUTATOR_H_
#define GLINITIALISERMUTATOR_H_

#include "GLBaseOrganismInitialiser.h"

class GLBaseMutator;
class GLBaseOrganism;

/**
 * This initialiser takes the arbitrary initial organism and 
 * mutator operator. Then each time when initialiseOrganism is called it
 * returns initial organism to which the mutato operator was applied.
 * It is recommened to assign mutator whith mutation rate 1.0 to guarantee
 * that mutation was really performed on the initial organism.
 */
class GLInitialiserMutator : public GLBaseOrganismInitialiser
{
public:
   /**
    * Constructor. Note, that copies of initialOrganism and mutator are
    * stored in the initialiser.
    * @param initialOrganism base organism for future mutations.
    * @param mutator mutation operator to be applied to base organism.
    *   it is recommended, that this operator had a mutation rate 1.0.
    * @param returnInitial when true the first time the initialiseOrganism
    *   is called the initial organism will be returned. 
    */
   GLInitialiserMutator(GLBaseOrganism *initialOrganism,
                        GLBaseMutator* mutator,
                        bool returnInitial = true);
   
   /**
    * Destructor. Destroys copies of initialOrganism and mutator stored
    * within this class.
    */
   virtual ~GLInitialiserMutator();
   
   /**
    * Creates and initialise organism.
    * @return pointer to the organism initialised with some initial value.
    */
   virtual GLBaseOrganism* initialiseOrganism();

   /**
    * Next call of initialiseOrganism after this function will return
    * the initial organisim.
    */
   void resetInitial();
   
private:
  GLBaseOrganism* m_initial_organism;
  GLBaseMutator* m_mutator;
  bool m_return_initial;
};

#endif /*GLINITIALISERMUTATOR_H_*/
