#include "GLInitialiserShuffleIntVector.h"
#include "GLLogicError.h"
#include "GLOrganismVector.h"
#include "GLRandomNumbersGenerator.h"

/**
 * Initialise organism. Organism must be of type
 * GLOrganismVector<int> and will be initialised with the 
 * vector of integers of size organism->getSize() from 0 to size
 * randomy shuffled.
 * @param organism organism to be initialised.
 * @return pointer to the same organism which was passed as a parameter,
 *            but now it is initialised.
 */
GLBaseOrganism* GLInitialiserShuffleIntVector::initialiseOrganism()
{
  //check, that the organism is of the correct type.
  vector<int> chromosome(m_size);
  GLRandomNumbersGenerator::shuffleIntegerVector(
          chromosome, 0, chromosome.size());
  return new GLOrganismVector<int>(chromosome);
}//initialiseOrganism


/**
 * Set the size for newly generated organisms.
 * @param size new size.
 */
void GLInitialiserShuffleIntVector::setSize(int size)
  {
    if (size <= 0)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLInitialiserShuffleIntVector::setSize\n"
                  "Parameter size(=%i) should be > 0",
                  size);
        throw errormsg;
      }
    m_size = size;
  }//setSize

