#ifndef GLINTERFACESWAPSECTION_H_
#define GLINTERFACESWAPSECTION_H_

/**
 * Interface for organisms which can swap sections between 2 genes.
 */
class GLInterfaceSwapSection
{
public:
  /**
   * Swaps sections with another organism. After this method is called
   * the current organism should have section of the anotherOrganism, while
   * the latter will have the section of the caller.
   * Therefore, all genes from the caller with numbers leftPos, rightPos - 1
   * inclusive will go to the anotherOrganism, while be replaced with
   * the corresponding genes from anotherOrganism in caller.
   * If rightPos == organism.getSize() the effect is equivalent to swapping
   * tails.
   * leftPos must be < rightPos.
   * @param anotherOrganism organism with which the section will be swapped.
   * @param leftPos the beginning of the section to be swapped (this gene
   *    will be swapped too).
   * @param rightPos the end of the section to be swapped (this gene will NOT
   *    be swapped).
   */
  virtual void swapSections(GLBaseOrganism* anotherOrganism, 
                            int leftPos, int rightPos) = 0;
};

#endif /*GLINTERFACESWAPSECTION_H_*/
