#include "GLIteratorStorageStl.h"
#include "GLRuntimeError.h"
/**
 * Constructor -- gets the pointer on the issuing class.
 * @param issuer pointer to the class which issued iterator.
 */
template <class TStlContainer, class TStlIterator, class TOrganism>
GLIteratorStorageStl<TStlContainer, TStlIterator, TOrganism>::
  GLIteratorStorageStl(TStlContainer& container): m_container(container)
{
  m_it = m_container.begin();
}

/**
 * Operator for moving forward along the sequence.
 * @return the pointer to itself.
 */
template <class TStlContainer, class TStlIterator, class TOrganism>
GLBidirectionalIterator<TOrganism>&
GLIteratorStorageStl<TStlContainer, TStlIterator, TOrganism>::
 operator++()
  {
    if (isEnd())
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLIteratorStorageStl::operator++\n"
                        "Cannot move iterator further -- end is reached\n");
        throw GLRuntimeError(errormsg);
      }
    m_it++;
    return *this;
  }//operator++

/**
 * Operator for moving backward along the sequence.
 * @return the pointer to itself.
 */
template <class TStlContainer, class TStlIterator, class TOrganism>
GLBidirectionalIterator<TOrganism>&
GLIteratorStorageStl<TStlContainer, TStlIterator, TOrganism>::
 operator--()
  {
    if (isBegin())
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLIteratorStorageStl::operator--\n"
                  "Cannot move iterator further -- beginning is reached\n");
        throw GLRuntimeError(errormsg);
      }
    m_it--;
    return *this;
  }//operator--

/**
 * Method to return the pointer to the element.
 * @return pointer to the element to which iterators points.
 */
template <class TStlContainer, class TStlIterator, class TOrganism>
TOrganism&
GLIteratorStorageStl<TStlContainer, TStlIterator, TOrganism>::
getElement()
  {
    if (isEnd())
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLIteratorStorageStl::getElement\n"
                        "Cannot return the pointer to organism when "
                        "iterator points to the end\n");
        throw GLRuntimeError(errormsg);
      }
    //Use const_cast, otherwise it would not work for the 'set' container.
    //Need seriously think if it is possible to redesign the access to
    //organisms [TODO]
    TOrganism& t = const_cast<TOrganism&>(*m_it);
    return t;
  }//getElement()

/**
 * Method to return the pointer to the element.
 * @return pointer to the element to which iterators points.
 */
template <class TStlContainer, class TStlIterator, class TOrganism>
const TOrganism&
GLIteratorStorageStl<TStlContainer, TStlIterator, TOrganism>::
getElement() const
  {
    if (isEnd())
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLIteratorStorageStl::getElement\n"
                        "Cannot return the pointer to organism when "
                        "iterator points to the end\n");
        throw GLRuntimeError(errormsg);
      }
    const TOrganism& t = (*m_it);
    return t;
  }//getElement()
