#ifndef GLITERATORSTORAGESTL_H_
#define GLITERATORSTORAGESTL_H_

#include <stdlib.h>

#include "GLIterators.h"

/**
 * Templated class for iterator over the stl container of organisms.
 */
template <class TStlContainer, class TStlIterator, class TOrganism>
class GLIteratorStorageStl : public GLBidirectionalIterator<TOrganism>
{
public:
  /**
   * Constructor -- gets the pointer on the issuing class.
   * @param container stl container with organisms.
   */
  GLIteratorStorageStl(TStlContainer& container);
  /**
   * Simple destructor.
   */
  virtual ~GLIteratorStorageStl() {}

  /**
   * Operator for moving forward along the sequence. Implementation
   * will depend on container.
   * @return the pointer to itself.
   */
  GLBidirectionalIterator<TOrganism>& operator++();

  /**
   * Operator for moving backward along the sequence. Implementation
   * will depend on container.
   * @return the pointer to itself.
   */
  GLBidirectionalIterator<TOrganism>& operator--();

  /**
   * Method to return the pointer to the element.
   * @return pointer to the element to which iterators points.
   */
  virtual TOrganism& getElement();

  /**
   * Constant method to return the pointer to the element.
   * @return pointer to the element to which iterators points.
   */
  virtual const TOrganism& getElement() const;

  /**
   * Returns true if iterator points to the first element in sequence.
   * @return true if iterator points to the first element in sequence.
   */
  virtual bool isBegin() const
  { return m_it == m_container.begin(); }

  /**
   * Returns true if iterator points beyond the last element in sequence.
   * Always true for empty sequences. If isEnd() == true the status of the
   * pointer is undefined.
   * @return true if iterator points beyond the last element in sequence.
   */
  virtual bool isEnd() const
    { return m_it == m_container.end(); }

  /**
   * Returns true if iterator points to NULL pointer and false otherwise.
   * @return true if iterator points to NULL pointer and false otherwise.
   */
  virtual bool isNull() const
  { return (*m_it) == NULL; }
  private:
      TStlContainer &m_container; //reference to container with organisms.
      TStlIterator m_it; //main iterator

};

#endif /* GLITERATORSTORAGE_H_ */
