#ifndef GLITERATORS_H_
#define GLITERATORS_H_

class GLBaseOrganism;

/**
 * Iterator classes in GLibrary.
 */
/**
 * Base class for moving through the organisms etc.
 */
template<class T> 
class GLIterator
{
public:
  /**
   * Creates new iterator.
   */
  GLIterator() {};
  
  /**
   * Destroys everything.
   */  
  virtual ~GLIterator() {};
  
  /**
   * Operator to return the pointer to the element.
   * @return pointer to the element to which iterators points.
   */
  T& operator*()
    { return getElement(); }
  
  /**
   * Operator to return the pointer to the element.
   * @return pointer to the element to which iterators points.
   */
  const T& operator*() const
    { return getElement(); }
  
  /**
   * Operator for moving forward along the sequence. Implementation 
   * will depend on container.
   * @return the pointer to itself.
   */
  virtual GLIterator<T>& operator++() = 0;
 
  /**
   * Method to return the pointer to the element.
   * @return pointer to the element to which iterators points.
   */
  virtual T& getElement() = 0;
  
  /**
   * Method to return the pointer to the element.
   * @return pointer to the element to which iterators points.
   */
  virtual const T& getElement() const = 0;
  
  /**
   * Returns true if iterator points to the first element in sequence.
   * @return true if iterator points to the first element in sequence.
   */
  virtual bool isBegin() const = 0;

  /**
   * Returns true if iterator points beyond the last element in sequence.
   * Always true for empty sequences. If isEnd() == true the status of the
   * pointer is undefined.
   * @return true if iterator points beyond the last element in sequence.
   */
  virtual bool isEnd() const = 0;
  
  /**
   * Returns true if iterator points to NULL pointer and false otherwise.
   * @return true if iterator points to NULL pointer and false otherwise.
   */
  virtual bool isNull() const = 0;

  /**
   * Moves forward along the sequence.
   * @return the pointer to itself.
   */
  virtual GLIterator* moveNext()
  { return &(++(*this)); }
};

/**
 * Base class for bidirectional iterator.
 */
template<class T> 
class GLBidirectionalIterator: public GLIterator<T>
{
public:
  /**
   * Creates new iterator.
   */
  GLBidirectionalIterator():
    GLIterator<T>()
    {}
  
  /**
   * Destroys everything.
   */
  virtual ~GLBidirectionalIterator() {};
  
  /**
   * Operator for moving backward along the sequence. Implementation 
   * will depend on container.
   * @return the pointer to itself.
   */
  virtual GLBidirectionalIterator<T>& operator--() = 0;
  
  /**
   * Moves backward along the sequence. 
   * @return pointer to itself.
   */
  virtual GLBidirectionalIterator* movePrev()
  { return &(--(*this)); }
};

typedef GLBidirectionalIterator<GLBaseOrganism*> 
                  GLBaseBidirectOrganismIterator;
typedef GLBidirectionalIterator<const GLBaseOrganism* const> 
                  GLBaseBidirectOrganismIteratorConst;
#endif /*GLBIDIRECTIONALITERATOR_H_*/
