#include <iostream>
#include <stdio.h>
#include <NLstl_utils.h>

#include "GLBaseOrganism.h"
#include "GLInterfaceMoveGene.h"
#include "GLMutatorGeneMove.h"
#include "GLRandomNumbersGenerator.h"

/**
 * Mutator. Passed pointer should point to the class implementing
 * GLMovableGeneInterface. 
 * Randomly chooses one gene and moves it to randomly chosen position
 * (it is guaranteed, that new position is different from the current
 * one unless the size of the organisms == 1).
 * @param organism pointer to the organism to be mutated.
 * @return true if organism was mutated and false otherwise.
 */
bool GLMutatorGeneMove::mutateOrganism(GLBaseOrganism* organism)
  {
    //Get the reference to the gene-movable interface.
    GLInterfaceMoveGene* m_organism = 
        nlutils::transform_pointer<GLInterfaceMoveGene,
         GLBaseOrganism>(organism,
                         "GLMutatorGenesSwapping::mutateOrganism\n"
                         "Passed pointer is either NULL or does not point "
                         "to the organism with GLInterfaceMoveGene");
    
    //See if random mutation event should occur
    if (GLRandomNumbersGenerator::getRandomUniform(0, 1) > 
                getMutationRate())
      {
        return false;
      }
    //check, that size >= 2, otherwise there is nowhere to move.
    if (organism->getSize() < 2)
      {
        return false;
      }
    //Ok, now we need to move gene in position.
    //Randomly choose the start position
    int first_index = GLRandomNumbersGenerator::
                        getRandomInteger(organism->getSize());
    //Then the second
    int second_index = GLRandomNumbersGenerator::
                        getRandomInteger(organism->getSize());
    //To make sure, that first and second indices do not coincide
    while(first_index == second_index)
      {
        second_index = GLRandomNumbersGenerator::
                            getRandomInteger(organism->getSize());
      }
    m_organism->moveGene(first_index, second_index);
    //printf("Exchanged indices %i and %i\n", first_index, second_index);
    return true;
  }
