#ifndef GLMUTATORGENEMOVE_H_
#define GLMUTATORGENEMOVE_H_

#include "GLBaseMutator.h"

/**
 * Mutates the organism by moving the gene from one position in organism
 * to another.
 */
class GLMutatorGeneMove : public GLBaseMutator
{
public:
  /**
   * Simple constructor.
   * @param mutationRate probability that gene will move. Should be
   *              in (0; 1) interval.
   */     
  GLMutatorGeneMove(double mutationRate = 0.5):
        GLBaseMutator(mutationRate, true){};
  
  /**
   * Copying constructor.
   * @param source where the data should be copied from.
   */
  GLMutatorGeneMove(const GLMutatorGeneMove& source):
    GLBaseMutator(source) {};

  /**
   * Destructor. Destroys everything.
   */
  virtual ~GLMutatorGeneMove() {}
  
  /**
   * Creates copy of the current exemplar of the mutator.
   * Should be overloaded in childrens.
   * @return copy of the current exemplar of the mutator.
   */
  virtual GLBaseMutator* copy() const
  { return new GLMutatorGeneMove(*this); }
  
  /**
   * Mutator. Passed pointer should point to the class implementing
   * GLMovableGeneInterface. 
   * Randomly chooses one gene and moves it to randomly chosen position
   * (it is guaranteed, that new position is different from the current
   * one unless the size of the organisms == 1).
   * @param organism pointer to the organism to be mutated.
   * @return true if organism was mutated and false otherwise.
   */
  virtual bool mutateOrganism(GLBaseOrganism* organism); 
};

#endif /*GLMUTATORGENEMOVE_H_*/
