#include <iostream>
#include <stdio.h>
#include <NLstl_utils.h>

#include "GLInterfaceExchangeableGenes.h"
#include "GLMutatorGenesSwapping.h"
#include "GLRandomNumbersGenerator.h"
#include "GLRuntimeError.h"
#include "gl_utils.h"
/**
 * Mutator. Passed pointer should point to the class implementing
 * GLExchengableGenesInterface. 
 * Randomly chooses 2 genes and swap them.
 * @param organism pointer to the organism to be mutated.
 * @param mutation_probability probability of the organism to be
 *            mutated.
 * @return true if organism was mutated and false otherwise.
 */
bool GLMutatorGenesSwapping::mutateOrganism(GLBaseOrganism* organism) 
 {
   //Get the reference to the exchangeable interface.
   GLInterfaceExchangeableGenes* x_organism = 
       nlutils::transform_pointer<GLInterfaceExchangeableGenes,
        GLBaseOrganism>(organism,
                        "GLMutatorGenesSwapping::mutateOrganism\n"
                        "Passed pointer is either NULL or does not point "
                        "to the organism with GLInterfaceExchangeableGenes");
   
   //See if random mutation event should occur
   if (GLRandomNumbersGenerator::getRandomUniform(0, 1) > 
               getMutationRate())
     {
       return false;
     }
   //check, that size >= 2, otherwise there is nothing to swap.
   if (organism->getSize() < 2)
     {
       return false;
     }
   //Ok, now we need to swap 2 elements.
   //Randomly choose the first one
   int first_index = GLRandomNumbersGenerator::
                       getRandomInteger(organism->getSize());
   int second_index = GLRandomNumbersGenerator::
                       getRandomInteger(organism->getSize());
   //To make sure, that first and second indices do not coincide
   while(first_index == second_index)
     {
       second_index = GLRandomNumbersGenerator::
                           getRandomInteger(organism->getSize());
     }
   x_organism->exchangeGenes(first_index, second_index);
   //printf("Exchanged indices %i and %i\n", first_index, second_index);
   return true;
 }//mutateOrganism
