#ifndef GLMUTATORGENESSWAPPING_H_
#define GLMUTATORGENESSWAPPING_H_

#include "GLBaseMutator.h"
#include "GLBaseOrganism.h"
/**
 * Class, which performs mutation based on the swapping pair of genes.
 * Requires, that class which is passed to it was implementing
 * ExchangableGenesInterface.
 */
class GLMutatorGenesSwapping : public GLBaseMutator
{
public:
/**
 * Simple constructor.
 * @param mutationRate probability that 2 gene will be swapped. Should be
 *              in (0; 1) interval.
 */	
  GLMutatorGenesSwapping(double mutationRate = 0.5):
      GLBaseMutator(mutationRate, true){};
  
  /**
   * Copying constructor.
   * @param source where the data should be copied from.
   */
  GLMutatorGenesSwapping(const GLMutatorGenesSwapping& source):
    GLBaseMutator(source) {};
  
  /**
   * Destructor, destroys everything.
   */
  virtual ~GLMutatorGenesSwapping(){};
  
  /**
   * Creates copy of the current exemplar of the mutator.
   * Should be overloaded in childrens.
   * @return copy of the current exemplar of the mutator.
   */
  virtual GLBaseMutator* copy() const
  { return new GLMutatorGenesSwapping(*this); }
  
  /**
   * Mutator. Passed pointer should point to the class implementing
   * GLExchengableGenesInterface. 
   * Randomly chooses 2 genes and swap them.
   * @param organism pointer to the organism to be mutated.
   * @return true if organism was mutated and false otherwise.
   */
  virtual bool mutateOrganism(GLBaseOrganism* organism); 
 };

#endif /*GLMUTATORGENESSWAPPING_H_*/
