#include "GLConstants.h"
#include "GLParametersGeneticAlgorithm.h"

static const int INITIAL_POPULATION_SIZE = 40;
static const double DEFAULT_MUTATION_RATE = 0.5;
static const int MAXIMUM_MUTANTS_PER_GENERATION = 50;
static const int MAXIMUM_CHILDREN_PER_GENERATION = 30;

/**
 * Copying constructor.
 * @param source where the data are copied from.
 */
GLParametersGeneticAlgorithm::GLParametersGeneticAlgorithm(
          const GLParametersGeneticAlgorithm* source):
   //copy main data
   m_breeder_params(NULL),
   m_breeder_type(source->m_breeder_type),
   m_children_per_generation(source->m_children_per_generation),
   m_crossover_params(NULL),
   m_crossover_type(source->m_crossover_type),
   m_initial_population_size(source->m_initial_population_size),
   m_keep_last_generation(source->m_keep_last_generation),
   m_maximum_population_size(source->m_maximum_population_size),
   m_minimum_population_size(source->m_minimum_population_size),
   m_mutants_per_generation(source->m_mutants_per_generation),
   m_mutation_policy(source->m_mutation_policy),
   m_mutation_rate(source->m_mutation_rate),
   m_mutator_params(NULL),
   m_mutator_type(source->m_mutator_type),
   m_other_params(NULL),
   m_population_params(NULL),
   m_ranker_params(NULL),
   m_ranker_type(source->m_ranker_type),
   m_selectionist_params(NULL),
   m_selectionist_type(source->m_selectionist_type),
   m_variator_params(NULL),
   m_variator_type(source->m_variator_type)
 {
    //copy additional parameters
    if (source->m_breeder_params != NULL)
      m_breeder_params            = source->m_breeder_params->copy();
    if (source->m_crossover_params != NULL)
    m_crossover_params            = source->m_crossover_params->copy();
    if (source->m_mutator_params != NULL)
      m_mutator_params            = source->m_mutator_params->copy();
    if (source->m_other_params != NULL)
      m_other_params              = source->m_other_params->copy();
    if (source->m_population_params != NULL)
      m_population_params         = source->m_population_params->copy();
    if (source->m_ranker_params != NULL)
      m_ranker_params             = source->m_ranker_params->copy();
    if (source->m_selectionist_params != NULL)
      m_selectionist_params       = source->m_selectionist_params->copy();
    if (source->m_variator_params != NULL)
      m_variator_params           = source->m_variator_params->copy();
  }//constructor

/**
 * Destructor -- destroys everything worth destroying.
 */
GLParametersGeneticAlgorithm::~GLParametersGeneticAlgorithm()
  {
    if (m_breeder_params != NULL)       delete m_breeder_params;
    if (m_crossover_params != NULL)     delete m_crossover_params;
    if (m_mutator_params != NULL)       delete m_mutator_params;
    if (m_other_params != NULL)         delete m_other_params;
    if (m_population_params != NULL)    delete m_population_params;
    if (m_ranker_params != NULL)        delete m_ranker_params;
    if (m_selectionist_params != NULL)  delete m_selectionist_params;
    if (m_variator_params != NULL)      delete m_variator_params;
  }//destructor

/**
 * Sets default values of the parameters.
 */
void GLParametersGeneticAlgorithm::setDefaultValues()
  {
    //set main parameters
    m_breeder_type = GLConstants::BREEDER_STANDARD;
    m_children_per_generation = MAXIMUM_CHILDREN_PER_GENERATION;
    m_crossover_type = GLConstants::CROSSOVER_VECTOR_KEEP_MATCHING_INT;
    m_initial_population_size = INITIAL_POPULATION_SIZE;
    m_keep_last_generation = false;
    m_maximum_population_size = m_initial_population_size;
    m_minimum_population_size = m_initial_population_size;
    m_mutants_per_generation = MAXIMUM_MUTANTS_PER_GENERATION;
    m_mutation_policy = GLConstants::MUTATE_PARENTS | 
                        GLConstants::MUTATE_CHILDREN;
    m_mutation_rate = DEFAULT_MUTATION_RATE;
    m_mutator_type = GLConstants::MUTATOR_GENES_SWAPPING;
    m_ranker_type = GLConstants::RANKER_STANDARD_SO;
    m_selectionist_type = GLConstants::SELECTIONIST_ELITIST;
    m_variator_type = GLConstants::VARIATOR_STANDARD;

    //set additional parameters to NULL
    m_breeder_params = NULL;
    m_crossover_params = NULL;
    m_mutator_params = NULL;
    m_other_params = NULL; 
    m_population_params = NULL;
    m_ranker_params = NULL;
    m_selectionist_params = NULL;
    m_variator_params = NULL;
  }//setDefaultValues
