#ifndef GLPARAMETERSGENETICALGORITHM_H_
#define GLPARAMETERSGENETICALGORITHM_H_

#include "GLBaseParameters.h"
#include "GLConstants.h"
/**
 * Structure with parameters common for all the genetic algorithms. As in
 * other children of GLBaseParameters all its members are public.
 */
struct GLParametersGeneticAlgorithm:public GLBaseParameters
{
public:
  /**
   * Constructor, sets default values for all parameters.
   */
  GLParametersGeneticAlgorithm()
  { setDefaultValues(); }

  /**
   * Copying constructor.
   * @param source where the data are copied from.
   */
  GLParametersGeneticAlgorithm(const GLParametersGeneticAlgorithm* source);

  /**
   * Destructor -- destroys everything worth destroying.
   */
  virtual ~GLParametersGeneticAlgorithm();

  /**
   * Copying operator -- makes copy of the parameters data.
   * @return pointer to GLParameters.
   */
  virtual GLBaseParameters* copy() const
  { return new GLParametersGeneticAlgorithm(this); }

  /**
   * Sets default values of the parameters.
   */
  void setDefaultValues();
public:
    ///additional parameters for breeder.
  GLBaseParameters*     m_breeder_params;
    ///type of the breeding operator.
  int                   m_breeder_type;
    ///number of children per generation
  int                   m_children_per_generation;
    ///additional params for crossover.
  GLBaseParameters*     m_crossover_params;
    ///type of the crossover operator.
  int                   m_crossover_type;
    ///initial population size.
  int                   m_initial_population_size;
    ///if true the last generation is stored in GA -- the one which was
    ///active just after GA finished work.
  bool                  m_keep_last_generation;
    ///maximum population size.
  int                   m_maximum_population_size;
    ///minimum population size.
  int                   m_minimum_population_size;
    ///number of mutants per generation
  int                   m_mutants_per_generation;
    ///determins to which organisms  mutation will be applied.
  int                   m_mutation_policy;
    ///mutation rate for mutator operator.
  double                m_mutation_rate;
    ///additional parameters for mutator.
  GLBaseParameters*     m_mutator_params;
    ///type of the mutator operator.
  int                   m_mutator_type;
    ///other (non-standard) parameters for GA
  GLBaseParameters*     m_other_params;
    ///additional params for population
  GLBaseParameters*     m_population_params;
    ///type of population.
  int                   m_population_type;
    ///additional parameters for ranker.
  GLBaseParameters*     m_ranker_params;
    ///type of the ranker operator.
  int                   m_ranker_type;
    ///additional parameters for selectionist
  GLBaseParameters*     m_selectionist_params;
    ///type of the selectionist operator.
  int                   m_selectionist_type;
  ///additional parameters for variator.
  GLBaseParameters*     m_variator_params;
    ///type of the variator operator.
  int                   m_variator_type;
};

#endif /*GLPARAMETERSGENETICALGORITHM_H_*/
