#ifndef GLSELECTIONISTSTANDARD_H_
#define GLSELECTIONISTSTANDARD_H_

#include "GLBaseSelectionist.h"

/**
 * Standard selectionist -- merges all the organisms in single pool,
 * sorts them according to fitness value and then chooses the best
 * to move to the next generation.
 */
class GLSelectionistStandard : public GLBaseSelectionist
{
public:
 /**
   * Constructor. Assigns the operator which will be used for ranking
   * the organisms. Note, that the ranker will be destroyed when destructor
   * is called.
   * @param ranker operator for arranging the organisms in accordance
   * 	with their rank.
  */
  GLSelectionistStandard(GLBaseRanker* ranker):GLBaseSelectionist(ranker)
		  {};

  /**
   * Destructor. Destroys everything.
   */
  virtual ~GLSelectionistStandard() {};

  /**
   * Choose which organism to leave and which one to die in the next
   * generation. It is assumed, that all organisms passed as input parameters
   * were evaluated already.
   * @param current_population organisms from the this generation's
   *            population.
   * @param new_generation newly created organisms (e.g. mutants and/or
   *            children). It is desirable, that both arrays have not
   *            common organisms.
   * @param new_size size of the next generation.
   * @param to_live output array of organisms to live  (will be cleaned).
   * @param to_die output array of organisms to die in the next generation
   *     (will be cleaned).
   * @param best_survived best survived organism(s). May be single organism
   *     for single objective optimisation or the whole Pareto-front for
   *     multi-objective optimisation. The previous content of this
   *     array will be lost.
   * @return number of NEW organisms (not present in current generation)
   *             in next generation.
   */
  virtual int doSelection(
                      const vector<GLBaseOrganism*>& current_population,
                      const vector<GLBaseOrganism*>& new_generation,
                      size_t new_size,
                      vector<GLBaseOrganism*>& to_live,
                      vector<GLBaseOrganism*>& to_die,
                      vector<const GLBaseOrganism*>& best_survived);
};

#endif /*GLSELECTIONISTSTANDARD_H_*/
