#include "GLFitnessSingleObjective.h"
#include "GLOrganismVector.h"
#include "GLRuntimeError.h"
#include "GLTestEvaluator.h"

/**
 * Simple constructor.
*/
GLTestEvaluator::GLTestEvaluator():
      GLBaseEvaluator()
{
}//constructor

/**
 * Evaluates fitness function of the organism. Requires organism to
 * be pointer to GLOrganismVector<int>. Returns the sum of absolute 
 * differences between neighbours in the vector of integers.
 * @param organism organism to be evaluated.
 * @return fitness value storage -- GLFitness<int>*.
 */
GLBaseFitness* GLTestEvaluator::evaluateOrganism(GLBaseOrganism* organism)
{
  GLOrganismVector<int> *org = dynamic_cast<GLOrganismVector<int>*>(organism);
  if (org == NULL)
    {
      char errormsg[1000];
      sprintf(errormsg, "\nGLTestEvaluator::evaluateOrganism\n"
                        "Passed organism must be valid pointer to "
                        "class GLOrganismVector<int>\n");
      throw GLRuntimeError(errormsg);
    }
  int sum = 0;
  //loop through all genes
  for (int i = 1; i < org->getSize(); i++)
    {
      sum += abs((*org)[i - 1] - (*org)[i]);
    }
  increaseCounter();
  return new GLFitnessSingleObjective<int>(sum);
}//evaluateOrganism
