#include <NLstl_utils.h>
#include <NLInterfaceOutput.h>

#include "GLBaseFitness.h"
#include "GLBaseOrganism.h"
#include "GLOrganismVector.h"
#include "GLLogicError.h"
#include "GLRuntimeError.h"
#include "GLUtils.h"

/**
 * organisms is not evaluated throws GLLogicError exception.
 * @param organism pointer to be asserted.
 * @param message useful information, e.g. name of the pointer and/or
 *            calling function.
 */
void GLUtils::assertEvaluated(const GLBaseOrganism *organism,
                              const string& message)
  {
    nlutils::assertPointer(organism, 
                           "from GLUtils::assertEvaluated " + message);
    if (!organism->isEvaluated())
      {
        char errormsg[1000];
        sprintf(errormsg, "\nassertEvaluated\n"
                          "%s organism is not evaluated\n",
                          message.c_str());
        throw GLLogicError(errormsg);
      }
  }//assertEvaluated

/**
 * Prints the vector of organisms pointers with their fitness value,
 * if present.
 * @param out output stream.
 * @param organisms
 */
void GLUtils::printVectorOfOrganisms(ostream& out,
                             const TGLVectorOfOrganisms& organisms)
  {
    for(unsigned i = 0; i < organisms.size(); i++)
      {
        nlutils::assertPointer(organisms[i],
            "organism[i] from GLUtils::printVectorOfOrganisms");
        out << i << " || " << flush;
        if (!organisms[i]->isEvaluated())
          {
            out << "Not evaluated ";
          }
        else
          {
            out << *(organisms[i]->getFitness());
          }
        out <<  " : " << *(organisms[i]) << endl;
      }
  }//printVectorOfOrganisms

/**
 * Prints the vector of organisms pointers with their fitness value,
 * if present.
 * @param out output stream.
 * @param organisms
 */
void GLUtils::printVectorOfOrganisms(ostream& out,
                             const TGLVectorOfOrganismsConst& organisms)
  {
    for(size_t i = 0; i < organisms.size(); i++)
      {
        nlutils::assertPointer(organisms[i],
            "organism[i] from GLUtils::printVectorOfOrganisms");
        out << i << " || " << flush;
        if (!organisms[i]->isEvaluated())
          {
            out << "Not evaluated ";
          }
        else
          {
            out << *(organisms[i]->getFitness());
          }
        out <<  " : " << *(organisms[i]) << endl;
      }
  }//printVectorOfOrganisms

