#ifndef GLUTILS_H_
#define GLUTILS_H_
#include <string>

#include <NLstl_utils.h>

#include "GLBaseOrganism.h"
/**
 * Class for simple utils, mostly static.
 */
class GLUtils
{
public:
  /**
   * Simple constructor.
   */
  GLUtils() {};

  /**
   * Simple destructor.
   */
  virtual ~GLUtils() {};

  /**
   * organisms is not evaluated throws GLLogicError exception.
   * @param organism pointer to be asserted.
   * @param message useful information, e.g. name of the pointer and/or
   *            calling function.
   */
  static void assertEvaluated(const GLBaseOrganism *organism,
                              const string& message = "");

  /**
   * Makes the hard copy of organisms in the vector (templated to allow
   * for processing constant and not-constant pointers to the organisms).
   * @param source vector of organisms to copy from.
   * @param target vector of organisms to copy to. All the previous content
   *    will be deleted and memory for organisms previously stored there
   *    will be freed.
   */
  template<class sourceOrg, class targetOrg>
  static void makeHardCopy(const vector<sourceOrg*>& source,
                           vector<targetOrg*>& target)
    {
      //clean the target
      for_each(target.begin(), target.end(),
               nlutils::DeleteObject());
      target.clear();
      target.reserve(source.size());
      transform(source.begin(),
                source.end(),
                back_inserter(target),
                GLBaseOrganism::Cloner(false));
    }
  /**
   * Prints the vector of organisms pointers with their fitness value,
   * if present.
   * @param out output stream.
   * @param organisms
   */
  static void printVectorOfOrganisms(ostream& out,
                               const TGLVectorOfOrganisms& organisms);

  /**
   * Prints the vector of organisms pointers with their fitness value,
   * if present.
   * @param out output stream.
   * @param organisms
   */
  static void printVectorOfOrganisms(ostream& out,
                               const TGLVectorOfOrganismsConst& organisms);

};
#endif /*GLUTILS_H_*/
