#ifndef GLVARIATORSTANDARD_H_
#define GLVARIATORSTANDARD_H_
#include "GLBaseVariator.h"

class GLBaseMutator;
class GLBaseOrganismStorage;
class GLBasePopulation;
/**
 * Class for performing standard variation -- apply mutator to all
 * organisms of the container.
 */
class GLVariatorStandard : public GLBaseVariator
{
public:
  /**
   * Constructor. Takes the pointer to mutator operator. Note, that this
   * mutator will be destroyed with this object.
   * @param mutator mutator operator.
   */
  GLVariatorStandard(GLBaseMutator *mutator);

  /**
   * Destroys the mutator operator (in base destructor).
   */
  virtual ~GLVariatorStandard();

  /**
   * Applies mutator to the set container of organisms.
   * @param population population to which mutated organisms belong -- mainly
   *    to check for uniqueness of the mutants.
   * @param organisms container with organisms.
   * @param mutants where mutated organisms will be added.
   * @param fill when equal to APPEND the container with mutants will be
   *    appended with new organisms, when REPLACE - mutants will be emptied
   *    at the beginning.
   * @param twins when NO_TWINS only unique organisms will be added.
   * @return number of new organisms produced by this method.
   * @param max_mutants maximum number of mutans which can be produced by
   *    mutator. When < 0 the number of mutants produced determined only
   *    by mutation rate.
   */
  virtual int applyMutator(GLBasePopulation* population,
                   GLBaseOrganismStorage* organisms,
                   GLBaseOrganismStorage* mutants,
                   GLConstants::TStorageFilling fill =
                     GLConstants::APPEND,
                   GLConstants::TTwinsFlags twins =
                     GLConstants::NO_TWINS,
                   int max_mutants = -1);

};

#endif /* GLVARIATORSTANDARD_H_ */
