#include <iomanip>
#include "NLTimer.h"
using namespace nlutils;
/**
 * Creates the timer class with optional message to be printed
 * every time the time value is printed.
 * @param msg message to be printed accompanying time.
 * @param printOnDestruction when true (default) will print the time
 *    on destruction of timer.
 */
NLTimer::NLTimer(const string& msg, bool print_on_destruction):
  m_message(msg), m_out(&cout),
  m_print_on_destruction(print_on_destruction)
{
  m_start = clock();
}//constructor

/**
 * Destroys the timer. If timer was created with default option will
 * print the time spent after the timer's creation.
 */
NLTimer::~NLTimer()
  {
    if (m_print_on_destruction)
      {
        printTime();
      }
  }//destructor

/**
 * Prints the time in seconds since the timer was created or reset.
 */
void NLTimer::printTime() const
{
  double current_time = (double)(clock() - m_start) / CLOCKS_PER_SEC;
  (*m_out) << m_message << " " << fixed << setprecision(2) 
           << current_time << endl;
}//printTime
