#ifndef NLTIMER_H_
#define NLTIMER_H_
#include <time.h>

#include <iostream>
#include <string>
using namespace std;

namespace nlutils
{
class NLTimer
{
public:
  /**
   * Creates the timer class with optional message to be printed
   * every time the time value is printed.
   * @param msg message to be printed accompanying time.
   * @param printOnDestruction when true (default) will print the time
   *    on destruction of timer.
   */
  NLTimer(const string& msg = "", bool print_on_destruction = true);
  
  /**
   * Destroys the timer. If timer was created with default option will
   * print the time spent after the timer's creation.
   */
  virtual ~NLTimer();
  
  /**
   * Returns time in seconds since timer started or have been reset.
   * @return time in seconds.
   */
  double getTimeSec() const
  { return (double)(clock() - m_start) / CLOCKS_PER_SEC; }

  /**
   * Prints the time in seconds since the timer was created or reset.
   */
  void printTime() const;
  
  /**
   * Resets the timer.
   */
  void reset()
    { m_start = clock(); }
  
  /**
   * Sets the stream for time output.
   * @param output new stream.
   */
  void setOutputStream(ostream& output)
    { m_out = &output; }
  
private:
  string m_message; //message to be print when it is created
  ostream* m_out;    //stream for output (by default cout).
  bool m_print_on_destruction; //print on destruction flag.
  clock_t m_start;  //marks the time when the timer was created or reset.
};
}
#endif /*NLTIMER_H_*/
